@@ -9,24 +9,22 @@ use strict;
 
 use Graphics::Framebuffer;
 use List::Util qw(min max shuffle);
-use Time::HiRes qw(sleep time);
+use Time::HiRes qw(sleep time alarm);
 
 # use Data::Dumper::Simple; $Data::Dumper::Sortkeys=1;
 
 system('clear');
 $|++;
 
-my $file = 0;
 my $arg = join(' ', @ARGV);
 my $new_x;
 my $new_y;
-my $F;
-my $FR;
+our $F;
+our $FR;
+my $DIRTY = 1;
 my $dev = 0;
 my $psize = 1;
 
-$file = 1 if ($arg =~ /file/i);    # File Handle drawing mode
-
 if ($arg =~ /X(\d+)/) {
     $new_x = $1;
     $arg =~ s/X\d+//;
@@ -51,46 +49,71 @@ our $DB = 0;
 our $STAMP = sprintf('%.1',time);
 
 if (defined($new_x)) {
-    $FR = Graphics::Framebuffer->new('FB_DEVICE' => "/dev/fb$dev", 'FILE_MODE' => $file, 'SHOW_ERRORS' => 1, 'SIMULATED_X' => $new_x, 'SIMULATED_Y' => $new_y, 'SPLASH' => 0);
+    $FR = Graphics::Framebuffer->new('FB_DEVICE' => "/dev/fb$dev", 'SHOW_ERRORS' => 0, 'SIMULATED_X' => $new_x, 'SIMULATED_Y' => $new_y,'SPLASH' => 0);
 } else {
-    $FR = Graphics::Framebuffer->new('FB_DEVICE' => "/dev/fb$dev", 'FILE_MODE' => $file, 'SHOW_ERRORS' => 1, 'SPLASH' => 0);
+    $FR = Graphics::Framebuffer->new('FB_DEVICE' => "/dev/fb$dev", 'SHOW_ERRORS' => 0, 'SPLASH' => 0);
 }
 
+$FR->cls('OFF');
+
 my $sinfo = $FR->screen_dimensions;
-if ($sinfo->{'bits_per_pixel'} == 16) {
-    $F = Graphics::Framebuffer->new('FB_DEVICE' => 'virtual', 'FILE_MODE' => $file, 'SHOW_ERRORS' => 1, 'SPLASH' => 0, 'VXRES' => $sinfo->{'width'}, 'VYRES' => $sinfo->{'height'}, 'COLOR_ORDER' => $sinfo->{'color_order'});
+# If 16 bit mode and acceleration is supported, then do double-buffering
+if ($sinfo->{'bits_per_pixel'} == 16 && $FR->{'ACCELERATED'}) {
+    $F = Graphics::Framebuffer->new('FB_DEVICE' => 'virtual', 'SHOW_ERRORS' => 1, 'SPLASH' => 0, 'VXRES' => $sinfo->{'width'}, 'VYRES' => $sinfo->{'height'}, 'COLOR_ORDER' => $sinfo->{'color_order'});
+    $F->{'fscreeninfo'}->{'id'} = $FR->{'fscreeninfo'}->{'id'};
     $DB = 1;
 } else {
     $F = $FR;
 }
 
-my $screen_width  = $F->{'XRES'};
-my $screen_height = $F->{'YRES'};
+my $screen_width  = $sinfo->{'width'};
+my $screen_height = $sinfo->{'height'};
 
 # Everything is based on a 1920x1080 screen, but different resolutions
 # are mathematically scaled.
-my $xm       = $F->{'XRES'} / 1920;
-my $ym       = $F->{'YRES'} / 1080;
+my $xm       = $screen_width / 1920;
+my $ym       = $screen_height / 1080;
 my $XX       = $screen_width;
 my $YY       = $screen_height;
 my $center_x = $F->{'X_CLIP'} + ($F->{'W_CLIP'} / 2);
 my $center_y = $F->{'Y_CLIP'} + ($F->{'H_CLIP'} / 2);
 my $factor   = 3;
-my $rpi      = ($F->{'fscreeninfo'}->{'id'} =~ /BCM270(8|9)/i) ? 1 : 0;
+my $rpi      = ($FR->{'fscreeninfo'}->{'id'} =~ /BCM270(8|9)/i) ? 1 : 0;
 
-# $factor *= 3 if ($rpi);    # Raspberry PI is sloooooow
+$factor *= 3 if ($rpi);    # Raspberry PI is sloooooow.  Let's give extra time for each test
 my $BW = 0;
+if ($rpi) {
+    print "Putting tennis balls on the walker, because this is a Raspberry PI\n";
+    sleep 3;
+}
+my $ALARM = ($rpi) ? 1/7.5 : 1/30; # Set double buffering flip timeout.  Raspberry PI has less FPS
+my $thread;
+if ($DB) { # If double buffering is on, then enable the flip alarm
+    $SIG{'ALRM'} = sub {
+        alarm(0);
+        if ($DIRTY) {
+            $FR->blit_flip($F);
+            $DIRTY = 0;
+        }
+        alarm($ALARM);
+    };
+    alarm($ALARM);
+}
+print_it($F, ' ','00FFFFFF');
+$F->splash($VERSION);
+my $LOGO = $F->blit_read(); # Get a memory copy of the splash screen
+
 foreach my $file (@files) {
     next if ($file =~ /^\.+/ || $file =~ /Test|gif/ || -d "$images_path/$file");
-    print_it($F, "Loading Images > $file",'00FFFFFF');
+    print_it($F, "Loading Images > $file",'00FFFFFF',undef,1);
     my $image = $F->load_image(
         {
             'x'            => 1,
             'y'            => 1,
-            'width'        => $XX * .90,
-            'height'       => $F->{'H_CLIP'} * .90,
+            'width'        => $XX,
+            'height'       => $F->{'H_CLIP'},
             'file'         => "$images_path/$file",
-            'convertalpha' => ($file =~ /wolf/i) ? 1 : 0,
+            'convertalpha' => ($file =~ /wolf|Crescent/i) ? 1 : 0,
             'center'       => CENTER_XY,
         }
     );
@@ -99,10 +122,10 @@ foreach my $file (@files) {
         {
             'x'            => 0,
             'y'            => 0,
-            'width'        => $XX * .4,
-            'height'       => $YY * .4,
+            'width'        => $XX * .5,
+            'height'       => $F->{'H_CLIP'} * .5,
             'file'         => "$images_path/$file",
-            'convertalpha' => ($file =~ /wolf/i) ? 1 : 0,
+            'convertalpha' => ($file =~ /wolf|Crescent/i) ? 1 : 0,
             'center'       => CENTER_XY,
         }
     );
@@ -110,15 +133,13 @@ foreach my $file (@files) {
     $BW = scalar(@IMAGES) - 1 if ($file =~ /wolf/i);
 } ## end foreach my $file (@files)
 
-$SIG{'QUIT'} = sub { $F->clip_reset(); $F->cls('ON'); exec('reset'); };
-$SIG{'INT'}  = sub { $F->clip_reset(); $F->cls('ON'); exec('reset'); };
 $F->wait_for_console(!$rpi);
 $F->cls('OFF');
-$F->splash($VERSION);
-update();
-sleep 1;
 
-# if (0) {
+$DIRTY = 1; # Set this to signal the buffer needs flipping
+sleep 4;
+
+# if (0) { # This line only ued for debugging
 color_mapping();
 plotting();
 foreach my $flag (0 .. 1) {
@@ -216,7 +237,8 @@ sub color_mapping {
             }
         }
     );
-    update();
+$DIRTY = 1;
+    
     sleep $factor / 2;
     
     $F->rbox(
@@ -236,7 +258,8 @@ sub color_mapping {
             }
         }
     );
-    update();
+    
+$DIRTY = 1;
     sleep $factor / 2 unless ($rpi);
     
     my $image = $F->load_image(
@@ -250,7 +273,8 @@ sub color_mapping {
         }
     );
     $F->blit_write($image);
-    update();
+$DIRTY = 1;
+    
     sleep $factor;
 } ## end sub color_mapping
 
@@ -264,7 +288,8 @@ sub plotting {
         my $y = int(rand($screen_height));
         $F->set_color({ 'alpha' => 255, 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->plot({ 'x' => $x, 'y' => $y, 'pixel_size' => $psize });
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
 } ## end sub plotting
 
@@ -280,7 +305,8 @@ sub lines {
     while (time < $s) {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->line({ 'x' => int(rand($XX)), 'y' => int(rand($YY)), 'xx' => int(rand($XX)), 'yy' => int(rand($YY)), 'antialiased' => $aa, 'pixel_size' => $psize });
-        update();
+$DIRTY = 1;
+        
     }
 } ## end sub lines
 
@@ -300,7 +326,7 @@ sub angle_lines {
         $F->angle_line({ 'x' => $center_x, 'y' => $center_y, 'radius' => int($F->{'H_CLIP'} / 2), 'angle' => $angle, 'antialiased' => $aa, 'pixel_size' => $psize });
 	    $angle += 7;
         $angle -= 360 if ($angle >= 360);
-        update();
+$DIRTY = 1;
     } ## end while (time < $s)
 } ## end sub angle_lines
 
@@ -312,7 +338,8 @@ sub boxes {
     while (time < $s) {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->box({ 'x' => int(rand($XX)), 'y' => int(rand($YY)), 'xx' => int(rand($XX)), 'yy' => int(rand($YY)), 'pixel_size' => $psize });
-        update();
+$DIRTY = 1;
+        
     }
 } ## end sub boxes
 
@@ -324,7 +351,8 @@ sub filled_boxes {
     while (time < $s) {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->box({ 'x' => int(rand($XX)), 'y' => int(rand($YY)), 'xx' => int(rand($XX)), 'yy' => int(rand($YY)), 'filled' => 1 });
-        update();
+$DIRTY = 1;
+        
     }
 } ## end sub filled_boxes
 
@@ -362,7 +390,8 @@ sub gradient_boxes {
                 }
             }
         );
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
 } ## end sub vertical_gradient_boxes
 
@@ -391,7 +420,8 @@ sub hatch_filled_boxes {
                 'hatch'  => $HATCHES[int(rand(scalar(@HATCHES)))]
             }
         );
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
     $F->attribute_reset();
 } ## end sub hatch_filled_boxes
@@ -413,7 +443,8 @@ sub texture_filled_boxes {
                 'texture' => $image
             }
         );
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
 } ## end sub texture_filled_boxes
 
@@ -425,7 +456,8 @@ sub rounded_boxes {
     while (time < $s) {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->box({ 'x' => int(rand($XX)), 'y' => int(rand($YY)), 'xx' => int(rand($XX)), 'yy' => int(rand($YY)), 'radius' => 4 + rand($XX / 16), 'pixel_size' => $psize });
-        update();
+$DIRTY = 1;
+        
     }
 } ## end sub rounded_boxes
 
@@ -437,7 +469,8 @@ sub filled_rounded_boxes {
     while (time < $s) {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->box({ 'x' => int(rand($XX)), 'y' => int(rand($YY)), 'xx' => int(rand($XX)), 'yy' => int(rand($YY)), 'radius' => 4 + rand($XX / 16), 'filled' => 1 });
-        update();
+$DIRTY = 1;
+        
     }
 } ## end sub filled_rounded_boxes
 
@@ -450,7 +483,8 @@ sub hatch_filled_rounded_boxes {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->set_b_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->box({ 'x' => int(rand($XX)), 'y' => int(rand($YY)), 'xx' => int(rand($XX)), 'yy' => int(rand($YY)), 'radius' => 4 + rand($XX / 16), 'filled' => 1, 'hatch' => $HATCHES[int(rand(scalar(@HATCHES)))] });
-        update();
+$DIRTY = 1;
+        
     }
     $F->attribute_reset();
 } ## end sub hatch_filled_rounded_boxes
@@ -492,7 +526,8 @@ sub gradient_rounded_boxes {
                 }
             }
         );
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
 } ## end sub vertical_gradient_rounded_boxes
 
@@ -514,7 +549,8 @@ sub texture_filled_rounded_boxes {
                 'texture' => $image
             }
         );
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
 } ## end sub texture_filled_rounded_boxes
 
@@ -526,7 +562,8 @@ sub circles {
     while (time < $s) {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->circle({ 'x' => int(rand($XX)), 'y' => int(rand($YY)), 'radius' => rand($center_y), 'pixel_size' => $psize });
-        update();
+$DIRTY = 1;
+        
     }
 } ## end sub circles
 
@@ -538,7 +575,8 @@ sub filled_circles {
     while (time < $s) {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->circle({ 'x' => int(rand($XX)), 'y' => int(rand($YY)), 'radius' => rand($center_y), 'filled' => 1 });
-        update();
+$DIRTY = 1;
+        
     }
 } ## end sub filled_circles
 
@@ -551,7 +589,8 @@ sub hatch_filled_circles {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->set_b_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->circle({ 'x' => int(rand($XX)), 'y' => int(rand($YY)), 'radius' => rand($center_y), 'filled' => 1, 'hatch' => $HATCHES[int(rand(scalar(@HATCHES)))] });
-        update();
+$DIRTY = 1;
+        
     }
     $F->attribute_reset();
 } ## end sub hatch_filled_circles
@@ -587,7 +626,8 @@ sub gradient_circles {
                 }
             }
         );
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
 } ## end sub vertical_gradient_circles
 
@@ -607,7 +647,8 @@ sub texture_filled_circles {
                 'texture' => $image
             }
         );
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
 } ## end sub texture_filled_circles
 
@@ -619,7 +660,8 @@ sub arcs {
     while (time < $s) {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->arc({ 'x' => int(rand($XX)), 'y' => int(rand($YY)), 'radius' => rand($center_y), 'start_degrees' => rand(360), 'end_degrees' => rand(360), 'pixel_size' => $psize });
-        update();
+$DIRTY = 1;
+        
     }
 } ## end sub arcs
 
@@ -630,7 +672,8 @@ sub poly_arcs {
     while (time < $s) {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->poly_arc({ 'x' => int(rand($XX)), 'y' => int(rand($YY)), 'radius' => rand($center_y), 'start_degrees' => rand(360), 'end_degrees' => rand(360), 'pixel_size' => $psize });
-        update();
+$DIRTY = 1;
+        
     }
 } ## end sub poly_arcs
 
@@ -642,7 +685,8 @@ sub filled_pies {
     while (time < $s) {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->filled_pie({ 'x' => int(rand($XX)), 'y' => int(rand($YY)), 'radius' => rand($center_y), 'start_degrees' => rand(360), 'end_degrees' => rand(360) });
-        update();
+$DIRTY = 1;
+        
     }
 } ## end sub filled_pies
 
@@ -655,7 +699,8 @@ sub hatch_filled_pies {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->set_b_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->filled_pie({ 'x' => int(rand($XX)), 'y' => int(rand($YY)), 'radius' => rand($center_y), 'start_degrees' => rand(360), 'end_degrees' => rand(360), 'hatch' => $HATCHES[int(rand(scalar(@HATCHES)))] });
-        update();
+$DIRTY = 1;
+        
     }
     $F->attribute_reset();
 } ## end sub hatch_filled_pies
@@ -690,7 +735,8 @@ sub gradient_pies {
                 }
             }
         );
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
 } ## end sub vertical_gradient_pies
 
@@ -715,7 +761,8 @@ sub texture_filled_pies {
                 'texture'       => $image,
             }
         );
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
 } ## end sub texture_filled_pies
 
@@ -727,7 +774,8 @@ sub ellipses {
     while (time < $s) {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->ellipse({ 'x' => int(rand($XX)), 'y' => int(rand($YY)), 'xradius' => rand($center_x), 'yradius' => rand($center_y), 'pixel_size' => $psize });
-        update();
+$DIRTY = 1;
+        
     }
 } ## end sub ellipses
 
@@ -739,7 +787,8 @@ sub filled_ellipses {
     while (time < $s) {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)), 'alpha' => int(rand(256)) });
         $F->ellipse({ 'x' => int(rand($XX)), 'y' => int(rand($YY)), 'xradius' => rand($center_x), 'yradius' => rand($center_y), 'filled' => 1 });
-        update();
+$DIRTY = 1;
+        
     }
 } ## end sub filled_ellipses
 
@@ -752,7 +801,8 @@ sub hatch_filled_ellipses {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->set_b_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->ellipse({ 'x' => int(rand($XX)), 'y' => int(rand($YY)), 'xradius' => rand($center_x), 'yradius' => rand($center_y), 'filled' => 1, 'hatch' => $HATCHES[int(rand(scalar(@HATCHES)))] });
-        update();
+$DIRTY = 1;
+        
     }
     $F->attribute_reset();
 } ## end sub hatch_filled_ellipses
@@ -786,28 +836,30 @@ sub gradient_ellipses {
                 }
             }
         );
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
 } ## end sub vertical_gradient_ellipses
 
 sub texture_filled_ellipses {
     
-    print_it($F, 'Image Filled Ellipse Test');
+    print_it($F, 'Texture Filled Ellipse Test');
 
-    my $s = ($F->{'BITS'} == 16) ? time + $factor * 3 : time + $factor * 2;
+    my $s = time + $factor;
     while (time < $s) {
         my $image = $IMAGES[int(rand(scalar(@IMAGES)))];
         $F->ellipse(
             {
                 'x'       => int(rand($XX)),
                 'y'       => int(rand($YY)),
-                'xradius' => rand($center_x),
-                'yradius' => rand($center_y),
+                'xradius' => int(rand($center_x)),
+                'yradius' => int(rand($center_y)),
                 'filled'  => 1,
                 'texture' => $image
             }
         );
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
 } ## end sub texture_filled_ellipses
 
@@ -835,7 +887,8 @@ sub polygons {
                 'pixel_size'  => $psize
             }
         );
-        update();
+$DIRTY = 1;
+        
     }
 } ## end sub polygons
 
@@ -857,7 +910,8 @@ sub filled_polygons {
                 'filled'      => 1,
             }
         );
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
 } ## end sub filled_polygons
 
@@ -882,7 +936,8 @@ sub hatch_filled_polygons {
                 'hatch'       => $HATCHES[int(rand(scalar(@HATCHES)))]
             }
         );
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
     $F->attribute_reset();
 } ## end sub hatch_filled_polygons
@@ -917,7 +972,8 @@ sub gradient_polygons {
                 }
             }
         );
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
 } ## end sub vertical_gradient_polygons
 
@@ -941,12 +997,12 @@ sub texture_filled_polygons {
                 'texture'     => $image
             }
         );
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
 } ## end sub texture_filled_polygons
 
 sub beziers {
-    
     print_it($F, 'Testing Bezier Curves');
 
     my $s = time + $factor;
@@ -957,7 +1013,8 @@ sub beziers {
             push(@coords, int(rand($XX)), int(rand($YY)));
         }
         $F->bezier({ 'coordinates' => \@coords, 'points' => 100, 'pixel_size' => $psize });
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
 } ## end sub beziers
 
@@ -992,7 +1049,8 @@ sub truetype_fonts {
             $b->{'x'} = rand($F->{'XX_CLIP'} - $b->{'pwidth'});
             $F->ttf_print($b);
         }
-        $STAMP = '';update();
+$DIRTY = 1;
+        
     } ## end while (time < $g)
 } ## end sub truetype_fonts
 
@@ -1028,7 +1086,8 @@ sub rotate_truetype_fonts {
         }
         $angle++;
         $angle = 0 if ($angle >= 360);
-        update();
+$DIRTY = 1;
+        
     } ## end while (time < $g)
 } ## end sub rotate_truetype_fonts
 
@@ -1038,7 +1097,7 @@ sub flood_fill {
     $F->clip_reset();
     my $image = $IMAGES[int(rand(scalar(@IMAGES)))];
 
-    if ($XX > 255) {
+    if ($XX > 255 && !$rpi) {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->polygon({ 'coordinates' => [220 * $xm, 190 * $ym, 1520 * $xm, 80 * $xm, 1160 * $xm, $YY, 960 * $xm, 540 * $ym, 760 * $xm, 780 * $ym] });
 
@@ -1047,7 +1106,8 @@ sub flood_fill {
 
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->circle({ 'x' => 500 * $xm, 'y' => 320 * $ym, 'radius' => 100 * $xm });
-        $STAMP = '';update();
+$DIRTY = 1;
+        
         sleep 1 if ($F->{'ACCELERATED'});
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
 
@@ -1056,19 +1116,21 @@ sub flood_fill {
         $F->{'ACCELERATED'} = 0;
         $F->fill({ 'x' => int(350 * $xm), 'y' => int(250 * $ym), 'texture' => $image });
         $F->{'ACCELERATED'} = $saved;
-        $STAMP = '';update();
+$DIRTY = 1;
+        
 
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->fill({ 'x' => 960 * $xm, 'y' => 440 * $ym });
     } else {
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->polygon({ 'coordinates' => [$center_x, 3, 3, $YY - 3, $center_x, $center_y, $XX - 3, $YY - 4] });
-        $STAMP = '';update();
+        
 
         $F->set_color({ 'red' => int(rand(256)), 'green' => int(rand(256)), 'blue' => int(rand(256)) });
         $F->fill({ 'x' => 3, 'y' => 3 });
     } ## end else [ if ($XX > 255) ]
-    $STAMP = '';update();
+$DIRTY = 1;
+    
     sleep $factor if ($F->{'ACCELERATED'});
 } ## end sub flood_fill
 
@@ -1078,7 +1140,7 @@ sub color_replace_with_clipping {
 
     my $x = $F->{'XRES'} / 4;
     my $y = $F->{'YRES'} / 4;
-    $F->splash($VERSION);
+    $F->blit_write($LOGO);
     $F->clip_set({ 'x' => $x, 'y' => $y, 'xx' => $x * 3, 'yy' => $y * 3 });
     my $r = 128;
     my $g = 0;
@@ -1103,7 +1165,8 @@ sub color_replace_with_clipping {
             }
         );
         ($r, $g, $b) = ($R, $G, $B);
-        $STAMP = '';update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
 } ## end sub color_replace_with_clipping
 
@@ -1111,7 +1174,7 @@ sub color_replace_no_clipping {
     
     print_it($F, 'Testing Color Replace (clipping off)');
 
-    $F->splash($VERSION);
+    $F->blit_write($LOGO);
 
     $F->clip_reset();
 
@@ -1138,7 +1201,8 @@ sub color_replace_no_clipping {
             }
         );
         ($r, $g, $b) = ($R, $G, $B);
-        $STAMP = '';update();
+$DIRTY = 1;
+        
     } ## end while (time < $s)
 } ## end sub color_replace_no_clipping
 
@@ -1153,7 +1217,8 @@ sub blitting {
         $image->{'x'} = abs(rand($XX - $image->{'width'}));
         $image->{'y'} = $F->{'Y_CLIP'} + abs(rand(($YY - $F->{'Y_CLIP'}) - $image->{'height'}));
         $F->blit_write($image);
-        $STAMP = '';update();
+$DIRTY = 1;
+        
     }
 } ## end sub blitting
 
@@ -1169,10 +1234,12 @@ sub blit_move {
     my $h = $image->{'height'};
     my $s = time + $factor;
     while (time < $s) {
-        $F->blit_move({'x' => $x,'y' => $y,'width' => $w, 'height' => $h, 'x_dest' => $x + 6,'y_dest' => $y + 3});
-        $x += 6;
-        $y += 3;
-        $STAMP = '';update();
+        $F->blit_move({'x' => abs($x),'y' => abs($y),'width' => $w, 'height' => $h, 'x_dest' => abs($x) + 4,'y_dest' => abs($y) + 2});
+        $x += 4;
+        $y += 2;
+        sleep (1/60);
+$DIRTY = 1;
+        
     }
 } ## end sub blit_move
 
@@ -1188,9 +1255,9 @@ sub rotate {
 
         my $st = 'Counter-Clockwise Rotate Test ';
         unless ($p) {
-            $st .= 'Using Standard Quslity Setting';
+            $st .= 'Using Standard Quality Setting';
         } else {
-            $st .= 'Using High Qualiuty Setting';
+            $st .= 'Using High Quality Setting';
         }
         print_it($F, $st);
         my $s = time + ($factor * 2);
@@ -1214,9 +1281,10 @@ sub rotate {
             );
 
             $F->blit_write($rot);
-            $STAMP = '';update();
+$DIRTY = 1;
+            
 
-            $angle += 5;
+            $angle += 3;
             $angle = 0 if ($angle >= 360);
             $count++;
         } ## end while (time < $s || $count...)
@@ -1250,11 +1318,12 @@ sub rotate {
             );
 
             $F->blit_write($rot);
+$DIRTY = 1;
 
-            $angle -= 5;
+            $angle -= 3;
             $angle = 0 if ($angle <= -360);
             $count++;
-            $STAMP = '';update();
+            
         } ## end while (time < $s || $count...)
         last if ($F->{'BITS'} == 16);
     } ## end foreach my $p (0 .. 1)
@@ -1280,9 +1349,14 @@ sub flipping {
             $rot->{'y'} = abs((($YY - $F->{'Y_CLIP'}) - $rot->{'height'}) / 2);
             print_it($F, "Testing Image Flip $dir");
             $F->blit_write($rot);
-            $STAMP = '';update();
+$DIRTY = 1;
+            
 
-            sleep .3 if (time < $zoom);
+            if (time < $zoom) {
+                sleep .3;
+            } else {
+                sleep (1/30); # Some framebuffers can't keep up
+            }
         } ## end foreach my $dir (qw(normal horizontal vertical both))
     } ## end while (time < $s)
 } ## end sub flipping
@@ -1300,7 +1374,7 @@ sub monochrome {
         $mono->{'x'} = abs(rand($XX - $mono->{'width'}));
         $mono->{'y'} = $F->{'Y_CLIP'} + abs(rand(($YY - $F->{'Y_CLIP'}) - $mono->{'height'}));
         $F->blit_write($mono);
-        $STAMP = '';update();
+        
     }
 } ## end sub monochrome
 
@@ -1341,9 +1415,9 @@ sub animated {
 
             foreach my $bench (0 .. 1) {
                 if ($count || $XX <= 320) {
-                    print_it($F, $bench ? "Playing Animated Image of '$info' Fullscreen Benchmark" : "Playing Animated Image of '$info' Fullscreen",$bench ? '0066FFFF' : 'FF00FFFF', $bench ? {'red' => 0, 'green' => 64, 'blue' => 0, 'alpha' => 255} : undef);
+                    print_it($F, $bench ? "Benchmarking Animated Image of '$info' Fullscreen" : "Playing Animated Image of '$info' Fullscreen",$bench ? '0066FFFF' : 'FF00FFFF', $bench ? {'red' => 0, 'green' => 64, 'blue' => 0, 'alpha' => 255} : undef);
                 } else {
-                    print_it($F, $bench ? "Playing Animated Image of '$info' Native Size Benchmark" : "Playing Animated Image of '$info' Native Size",$bench ? '0066FFFF' : 'FF00FFFF', $bench ? {'red' => 0, 'green' => 64, 'blue' => 0, 'alpha' => 255} : undef );
+                    print_it($F, $bench ? "Benchmarking Animated Image of '$info' Native Size" : "Playing Animated Image of '$info' Native Size",$bench ? '0066FFFF' : 'FF00FFFF', $bench ? {'red' => 0, 'green' => 64, 'blue' => 0, 'alpha' => 255} : undef );
                 }
                 if (defined($image)) {
                     $F->cls();
@@ -1360,7 +1434,8 @@ sub animated {
                             }
                             my $begin = time;
                             $F->blit_write($image->[$frame]);
-                            $STAMP = '';update();
+$DIRTY = 1;
+                            
                             my $delay = (($image->[$frame]->{'tags'}->{'gif_delay'} * .01)) - (time - $begin);
                             if ($delay > 0 && !$bench) {
                                 sleep $delay;
@@ -1399,12 +1474,14 @@ sub mode_drawing {
 
     $F->normal_mode();
     $F->blit_write($image);
-    $STAMP = '';update();
+$DIRTY = 1;
+    
     sleep 2;
 
     $F->{'DRAW_MODE'} = $mode;
     $F->blit_write($image2);
-    $STAMP = '';update();
+$DIRTY = 1;
+    
     sleep 2;
 
     my $size = int(($YY - $F->{'Y_CLIP'}) / 3);
@@ -1418,7 +1495,8 @@ sub mode_drawing {
 
     $F->set_color({ 'red' => 0, 'green' => 0, 'blue' => 255 });
     $F->circle({ 'x' => $mid + ($size / 2), 'y' => $F->{'Y_CLIP'} + $size * 2, 'radius' => $size, 'filled' => 1 });
-    $STAMP = '';update();
+$DIRTY = 1;
+    
     sleep $factor;
 } ## end sub or_drawing
 
@@ -1437,12 +1515,14 @@ sub alpha_drawing {
 
     $F->normal_mode();
     $F->blit_write($image);
-    $STAMP = '';update();
+$DIRTY = 1;
+    
     sleep 2;
 
     $F->alpha_mode();
     $F->blit_write($image2);
-    $STAMP = '';update();
+$DIRTY = 1;
+    
     sleep 2;
 
     $F->set_color({ 'red' => 0, 'green' => 255, 'blue' => 255, 'alpha' => int(rand(256)) });
@@ -1453,7 +1533,8 @@ sub alpha_drawing {
 
     $F->set_color({ 'red' => 255, 'green' => 255, 'blue' => 0, 'alpha' => int(rand(256)) });
     $F->rbox({ 'x' => ($XX / 2), 'y' => ($YY / 2), 'width' => ($XX / 2), 'height' => ($YY / 2), 'filled' => 1 });
-    $STAMP = '';update();
+$DIRTY = 1;
+    
     sleep $factor;
 } ## end sub alpha_drawing
 
@@ -1472,10 +1553,12 @@ sub mask_drawing {
     $F->normal_mode();
     $F->blit_write($image1);
     $F->mask_mode();
-    $STAMP = '';update();
+$DIRTY = 1;
+    
     sleep .3;
     $F->blit_write($image2);
-    $STAMP = '';update();
+$DIRTY = 1;
+    
     sleep $factor;
 } ## end sub mask_drawing
 
@@ -1492,10 +1575,12 @@ sub unmask_drawing {
     $F->normal_mode();
     $F->blit_write($image1);
     $F->unmask_mode();
-    $STAMP = '';update();
+$DIRTY = 1;
+    
     sleep .3;
     $F->blit_write($image2);
-    $STAMP = '';update();
+$DIRTY = 1;
+    
     sleep $factor;
 } ## end sub unmask_drawing
 
@@ -1504,14 +1589,15 @@ sub print_it {
     my $message = shift;
     my $color   = shift || 'FFFF00FF';
     my $bgcolor = shift || {'red' => 0, 'green' => 0, 'blue' => 0, 'alpha' => 0};
+    my $noclear = shift || 0;
 
     $fb->normal_mode();
-    $fb->clip_reset();
+#    $fb->clip_reset();
     $fb->set_b_color($bgcolor);
-    $fb->cls();
-    system('clear');
+    $fb->cls() unless ($noclear);
+    system('clear') unless($noclear);
     unless ($XX <= 320) {
-        $fb->or_mode();
+#        $fb->or_mode();
 
         my $b = $fb->ttf_print(
             {
@@ -1526,24 +1612,20 @@ sub print_it {
                 'antialias'    => 0
             }
         );
+        $fb->clip_set({ 'x' => 0, 'y'=> 0, 'yy' => $b->{'pheight'} * .75, 'xx' => $XX});
+        $fb->cls();
         $fb->ttf_print($b);
         $fb->clip_set({ 'x' => 0, 'y' => $b->{'pheight'} * .75, 'xx' => $XX, 'yy' => $YY });
-        $center_y = (($YY - $b->{'pheight'}) / 2) + $b->{'pheight'};
-        $fb->normal_mode();
+#        $center_y = (($YY - $b->{'pheight'}) / 2) + $b->{'pheight'};
+#        $fb->normal_mode();
     } else {
-        system('clear');
+        system('clear') unless ($noclear);
         print STDERR "$message\n";
     }
-    $STAMP = '';update();
+$DIRTY = 1;
+    
 } ## end sub print_it
 
-sub update {
-    my $now = time;
-    if ($now > $STAMP) {
-        $FR->blit_flip($F) if ($DB);
-        $STAMP = $now + (1 / 30);
-    }
-}
 
 __END__
 
