@@ -20,6 +20,10 @@ my @a = @ARGV;
 my $arg = join('', @a);
 my $Threads = $arg =~ /(\d+)/ ? $1 : Sys::CPU::cpu_count();
 
+$SIG{'ALRM'} = sub { exec('reset'); };
+$SIG{'QUIT'} = \&finish;
+$SIG{'INT'}  = \&finish;
+
 $file = 1 if ($arg =~ /file/i);    # File Handle mode
 
 print "\nRunning on a ", Sys::CPU::cpu_type(), " with $Threads threads\n";
@@ -31,7 +35,7 @@ my $two    = 1;
 $two = 2 if (-e "/dev/fb0" && -e "/dev/fb1");
 my $device = 0;
 foreach my $thr (0 .. $Threads) {
-    $framebuffer[$thr] = Graphics::Framebuffer->new('FILE_MODE' => $file, 'SPLASH' => $splash, 'FB_DEVICE' => sprintf('/dev/fb%d', $device));
+    $framebuffer[$thr] = Graphics::Framebuffer->new('FILE_MODE' => $file, 'SPLASH' => $splash, 'FB_DEVICE' => sprintf('/dev/fb%d', $device), 'RESET' => 0);
     if ($two == 2) {
         $device++;
         $device = 0 if ($device == 2);
@@ -41,30 +45,40 @@ foreach my $thr (0 .. $Threads) {
 my ($screen_width, $screen_height) = $framebuffer[0]->screen_dimensions();
 
 my $top = 16 * $Threads;    # Font height * # of threads
-
+my ($mw,$mh) = $framebuffer[0]->screen_dimensions();
+my $clw = $mw / ($Threads / $two);
+my $clx : shared = 0;
 foreach my $t (0 .. $Threads) {
     $framebuffer[$t]->cls();
 }
 cls;
 
-$SIG{'QUIT'} = \&finish;
-$SIG{'INT'}  = \&finish;
 my @thrd;
 foreach my $page (1 .. $Threads) {
     $waiting[$page] = 0;
     $thrd[$page]    = threads->create(
         sub {
+            local $SIG{'QUIT'} = undef; # sub { return; };
+            local $SIG{'INT'}  = undef; # sub { return; };
             my $Page = shift;
             my $top  = shift;
             my ($screen_width, $screen_height) = $framebuffer[$Page]->screen_dimensions();
+            if ($framebuffer[$Page]->{'FB_DEVICE'} eq '/dev/fb0') {
+                $framebuffer[$page]->clip_set({ 'x' => $clx, 'y' => $top, 'xx' => $clx + $clw, 'yy' => $screen_height });
+                {
+                    lock($clx);
+                    $clx += $clw;
+                }
+#            } else {
+#                $framebuffer[$page]->clip_set({ 'x' => 0, 'y' => $top, 'xx' => $screen_width, 'yy' => $screen_height });
+            }
             if (defined($framebuffer[$Page]->{'ERROR'})) {
-                print locate($Page, 1), clline, "ERROR: Thread $Page, $framebuffer[$Page]->{'ERROR'}" if ($screen_width > 320);
+                print locate($Page, 1), clline, "ERROR: Thread $Page, $framebuffer[$Page]->{'ERROR'}";
             }
-            $framebuffer[$page]->clip_set({ 'x' => 0, 'y' => $top, 'xx' => $screen_width, 'yy' => $screen_height }) if ($screen_width > 320);
             while ($RUNNING) {
                 attract($Page, $screen_width, $screen_height);
                 while ($hold && $RUNNING) {
-                    print locate($Page, 1), clline, colored(['white on_yellow'], "Thread $Page"), ' Waiting for other threads to finish drawing before clearing screen...' if ($screen_width > 320);
+                    print locate($Page, 1), clline, colored(['white on_yellow'], "Thread $Page"), ' Waiting for other threads to finish drawing ...';
                     threads->yield();
                     $waiting[$Page] = 0;
                     sleep 1;
@@ -75,9 +89,11 @@ foreach my $page (1 .. $Threads) {
                 select(STDOUT);
                 $| = 1;
             } ## end while ($RUNNING)
-            print locate($Page + 1, 1), clline, colored(['black on_green'], "Thread $Page"), ' Finished' if ($screen_width > 320);
+#            $framebuffer[$Page]->cls();
+            print locate($Page, 1), clline, colored(['black on_green'], "Thread $Page"), ' Finished';
             select(STDOUT);
             $| = 1;
+            return;
         },
         $page,
         $top
@@ -91,10 +107,11 @@ while (1) {
         while ($waiting[$page] && $thrd[$page]->is_running()) {
             sleep 1;
         }
+        $framebuffer[$page]->cls();
     }
-    foreach my $c (0 .. $Threads) {
-        $framebuffer[$c]->cls();
-    }
+#    foreach my $c (1 .. $Threads) {
+#        $framebuffer[$c]->cls();
+#    }
 } ## end while (1)
 
 ##############################################################################
@@ -111,16 +128,16 @@ sub attract {
     my $red           = int(rand(256));
     my $grn           = int(rand(256));
     my $blu           = int(rand(256));
-    my $x             = int(rand($screen_width));
+    my $x             = $framebuffer[$page]->{'X_CLIP'} + int(rand($framebuffer[$page]->{'W_CLIP'}));
     my $y             = int(rand($screen_height));
-    my $w             = int(rand($screen_width / 3));
+    my $w             = int(rand($framebuffer[$page]->{'W_CLIP'} / 3));
     my $h             = int(rand($screen_height / 3));
-    my $rx            = int(rand($screen_width / 5));
+    my $rx            = int(rand($framebuffer[$page]->{'W_CLIP'}));
     my $ry            = int(rand($screen_height / 5));
     my $sd            = int(rand(360));
     my $ed            = int(rand(360));
     my $gr            = (rand(10) / 10);
-    my $mode          = int(rand(4));
+    my $mode          = 0; # int(rand(4));
     my $type          = int(rand(13));
     my $size          = 1; # int(rand(6)) + 1;
     my $arc           = int(rand(3));
@@ -299,6 +316,7 @@ sub arctype {
 
 sub finish {
     $RUNNING = 0;
+    alarm 3;
     print locate(1, 1), "SHUTTING DOWN THREADS...";
     while(my @thr = threads->list(threads::running)) {
         sleep 1;
