@@ -63,7 +63,7 @@ sub _perl_logo {
             }
         )
     );
-} ## end sub _perl_logo
+}
 
 sub splash {
     my $self    = shift;
@@ -71,8 +71,8 @@ sub splash {
 
     my $X = $self->{'X_CLIP'};
     my $Y = $self->{'Y_CLIP'};
-    my $W = $self->{'XX_CLIP'} - $X;
-    my $H = $self->{'YY_CLIP'} - $Y;
+    my $W = $self->{'W_CLIP'};
+    my $H = $self->{'H_CLIP'};
 
     my $hf = $W / 1920;    # Scales the logo
     my $vf = $H / 1080;
@@ -177,7 +177,7 @@ sub splash {
                     'height'       => 40 * $vf,
                     'wscale'       => 1,
                     'color'        => 'FFFFFFFF',
-                    'text'         => uc($self->{'fscreeninfo'}->{'id'} || 'Unknown GPU'),
+                    'text'         => uc($self->{'GPU'}),
                     'bounding_box' => TRUE,
                     'center'       => $self->{'CENTER_X'},
                     'antialias'    => FALSE,
@@ -268,7 +268,7 @@ sub splash {
             {
                 'x'        => (200 * $hf) + $X,
                 'y'        => (200 * $vf) + $Y,
-                'radius'   => 199 * min($hf, $vf),
+                'radius'   => 198 * min($hf, $vf),
                 'filled'   => TRUE,
                 'gradient' => {
                     'direction' => 'horizontal',
@@ -465,7 +465,7 @@ sub splash {
                     }
                 )
             );
-        } ## end if ($self->{'ACCELERATED'...})
+        }
         if ($self->{'BITS'} >= 24) {
             my $shadow = $self->ttf_print(
                 {
@@ -490,7 +490,7 @@ sub splash {
             $shadow->{'y'} += max(1, 6 * $vf);
             delete($shadow->{'center'});
             $self->ttf_print($shadow);
-        } ## end if ($self->{'BITS'} >=...)
+        }
         my $gfb = $self->ttf_print(
             {
                 'bounding_box' => TRUE,
@@ -553,7 +553,7 @@ sub splash {
                 'height'       => 120 * $vf,
                 'wscale'       => 1,
                 'color'        => 'FFFFFFFF',
-                'text'         => sprintf('%dx%d-%02d on %s', $self->{'XRES'}, $self->{'YRES'}, $self->{'BITS'}, $self->{'fscreeninfo'}->{'id'} || 'Unknown GPU'),
+                'text'         => sprintf('%dx%d-%02d on %s', $self->{'XRES'}, $self->{'YRES'}, $self->{'BITS'}, $self->{'GPU'}),
                 'bounding_box' => TRUE,
                 'center'       => $self->{'CENTER_X'},
                 'antialias' => ($self->{'BITS'} >= 24) ? TRUE : FALSE
@@ -566,9 +566,9 @@ sub splash {
         }
         $self->ttf_print($scaleit);
         $self->_perl_logo();
-    } ## end else [ if ((($W <= 320) || ($H...)))]
+    }
     $self->normal_mode();
-} ## end sub splash
+}
 
 1;
 
